<?php
// (c) Juergen Galupki  http://galupki.de

class sonnemond {

// calculates basic astronomic events
// with low accuracy for general purpose
// results are given back into VARs...
// more documentation at the end of the file


// function sun:
// time of sunrise or sunset for a given day and given location
var $sun_hh = 0;                // 0-23
var $sun_mm = 0;                // 0-59

// function moon:
// phase of the moon
var $moonage = 0;               // 0-29 days (0+29 new moon, 14+15 full moon...)
var $moonvisible = 0;           // 0-100%
var $moonphase = "";            // in german language


function sun($was, $jahr, $monat, $tag, $zenith, $lon, $lat, $diff) {
# sunrise or sunset (+/- 5 minutes or so, mostly difference <2 min.)
# returns error-rc= 0(ok, valid)
#                   problems: 1(date) 2(zenith) 3(lon) 4(lat) 5(diff) 6(no rise this day) 7(no setting this day)
# input:
# was     "rising" or "setting"
# tag     dd
# monat   mm
# jahr    yyyy
# zenith  sun behind horizont 90.8333=official 96=civil 102=nautical 108=astronomical
# lon     longitude lnge decimal
# lat     latitude breite decimal
# diff    difference local time to UTC (GMT)


#0. basic check

   $rc = 0;

   if ( $was{0}=="r" ) {
      $was = "rising";
   } else {
      $was = "setting";
   }

   if ( !checkdate( $monat, $tag, $jahr) ) {
      $rc = 1;
   }

   if ( $zenith < 90.0  and  $zenith > 108.0 ) {
      $rc = 2;
      $zenith = 90.8333;
   }

   if ( $lon < -180.0 ) {
      $rc = 3;
      $lon = -180.0;
   }
   if ( $lon > 180.0 ) {
      $rc = 3;
      $lon = 180.0;
   }

   if ( $lat < -180.0 ) {
      $rc = 4;
      $lat = -180.0;
   }
   if ( $lat > 180.0 ) {
      $rc = 4;
      $lat = 180.0;
   }

   if ( $diff < -12.0 ) {
      $rc = 5;
      $diff = -12.0;
   }
   if ( $diff > 12.0 ) {
      $rc = 5;
      $diff = 12.0;
   }


#1. first calculate the day of the year

   $N1 = floor(275 * $monat / 9);
   $N2 = floor(($monat + 9) / 12);
   $N3 = (1 + floor(($jahr - 4 * floor($jahr / 4) + 2) / 3));
   $N = $N1 - ($N2 * $N3) + $tag - 30;

#2. convert the longitude to hour value and calculate an approximate time

   $lngHour = $lon / 15;
   if ($was=="rising") {
      $t = $N + ((6 - $lngHour) / 24);
   }
   else {
      $t = $N + ((18 - $lngHour) / 24);
   }

#3. calculate the Sun's mean anomaly

   $M = (0.9856 * $t) - 3.289;

#4. calculate the Sun's true longitude

   $L = $M + (1.916 * sin($M/180.0 * pi())) + (0.02 * sin(2 * $M / 180.0 * pi() ) ) + 282.634;
#  NOTE: L potentially needs to be adjusted into the range [-360,360) by adding/subtracting 360
   if ($L < -360) {
      $L = $L + 360;
   }
   if ($L > 360) {
      $L = $L - 360;
   }

#5a. calculate the Sun's right ascension

   $RA = atan(0.91764 * tan($L/180.0 * pi() )) * 180.0 / pi() ;
#  NOTE: RA potentially needs to be adjusted into the range [-360,360) by adding/subtracting 360
   if ($RA < -360) {
      $RA = $RA + 360;
   }
   if ($RA > 360) {
      $RA = $RA - 360;
   }

#5b. right ascension value needs to be in the same quadrant as L

   $Lquadrant  = (floor( $L/90.0)) * 90.0;
   $RAquadrant = (floor($RA/90.0)) * 90.0;
   $RA = $RA + ($Lquadrant - $RAquadrant);

#5c. right ascension value needs to be converted into hours

   $RA = $RA / 15;

#6. calculate the Sun's declination

   $sinDec = 0.39782 * sin($L/180.0 * pi() );
   $cosDec = cos(asin($sinDec));

#7a. calculate the Sun's local hour angle

   $cosH = (cos($zenith/180.0 * pi() ) - ($sinDec * sin($lat/180.0 * pi() ))) / ($cosDec * cos($lat/180.0 * pi() ));
   if ($cosH >  1) {
#     the sun never rises on this location (on the specified date)
      $rc = 6;
   }
   if ($cosH < -1) {
#     the sun never sets on this location (on the specified date)
      $rc = 7;
   }

#7b. finish calculating H and convert into hours

   if ($was=="rising") {
      $H = 360 - acos($cosH) * 180.0 / pi() ;
   }
   else {
      $H = acos($cosH) * 180.0 / pi() ;
   }
   $H = $H / 15.0;

#8. calculate local mean time of rising/setting

   $T = $H + $RA - (0.06571 * $t) - 6.622;

#9. adjust back to UTC

   $UT = $T - $lngHour;
#  NOTE: UT potentially needs to be adjusted into the range [0,24) by adding/subtracting 24
   if ($UT < 0.0) {
      $UT = $UT + 24;
   }
   if ($UT > 24) {
      $UT = $UT - 24;
   }

#10. convert UT value to local time zone of latitude/longitude

   $localT = $UT + $diff;

#11. split to std:min

   $this->sun_hh = floor($localT);
   $this->sun_mm = floor(($localT - $this->sun_hh) * 60);

   return $rc;

 } // end function sun




 function moon($year, $month, $day) {
# reports lunar phase (+/- 1 day or so)
# moonphase as string in german language
# returns error-rc= 0(ok) or problem: 1(date)

   $rc = 0;
   if ( !checkdate( $month, $day, $year) ) {
      $rc = 1;
      $year  = strftime("%Y");
      $month = strftime("%m");
      $day   = strftime("%d");
   }


   $x = ( $year - floor($year / 100) * 100 ) % 19;
   if ($x>9) {
      $x = $x-19;
   }
   $x = ($x*11) % 30;
   $x = $x + $month + $day;
   if ($month<3) {
      $x = $x+2;
   }
   $x = $x - 4;
   if ($year>1999) {
      $x = $x-4.3;
   }

   $this->moonage = round($x + 30) % 30;
   $this->moonvisible = abs(round(100 - abs($this->moonage - 14) * 100 / 14));

   if ( $this->moonage==14 ) {
     $this->moonphase = "Vollmond";
   } else if ( $this->moonage ==0 ) {
     $this->moonphase = "Neumond";
   } else if ( $this->moonage<14 ) {
     $this->moonphase = "zunehmend, " . (14-$this->moonage) . " T. bis Vollmond";
   } else {
     $this->moonphase = "abnehmend, " . (30-$this->moonage) . " T. bis Neumond";
   }

   return $rc;

 } // end function moon


} // end class sonne


/* **************************************************************************

Sunrise/Sunset Algorithm

Source:
        Almanac for Computers, 1990
        published by Nautical Almanac Office
        United States Naval Observatory
        Washington, DC 20392

Inputs:
        day, month, year:      date of sunrise/sunset
        latitude, longitude:   location for sunrise/sunset
        zenith:                Sun's zenith for sunrise/sunset
          offical      = 90 degrees 50'
          civil        = 96 degrees
          nautical     = 102 degrees
          astronomical = 108 degrees

        NOTE: longitude is positive for East and negative for West

        Lnge 7,2
        Breite 50,767
        Differenz zu UT (1/2


1. first calculate the day of the year

        N1 = floor(275 * month / 9)
        N2 = floor((month + 9) / 12)
        N3 = (1 + floor((year - 4 * floor(year / 4) + 2) / 3))
        N = N1 - (N2 * N3) + day - 30

2. convert the longitude to hour value and calculate an approximate time

        lngHour = longitude / 15

        if rising time is desired:
          t = N + ((6 - lngHour) / 24)
        if setting time is desired:
          t = N + ((18 - lngHour) / 24)

3. calculate the Sun's mean anomaly

        M = (0.9856 * t) - 3.289

4. calculate the Sun's true longitude

        L = M + (1.916 * sin(M)) + (0.020 * sin(2 * M)) + 282.634
        NOTE: L potentially needs to be adjusted into the range [-360,360) by adding/subtracting 360

5a. calculate the Sun's right ascension

        RA = atan(0.91764 * tan(L))
        NOTE: RA potentially needs to be adjusted into the range [-360,360) by adding/subtracting 360

5b. right ascension value needs to be in the same quadrant as L

        Lquadrant  = (floor( L/90)) * 90
        RAquadrant = (floor(RA/90)) * 90
        RA = RA + (Lquadrant - RAquadrant)

5c. right ascension value needs to be converted into hours

        RA = RA / 15

6. calculate the Sun's declination

        sinDec = 0.39782 * sin(L)
        cosDec = cos(asin(sinDec))

7a. calculate the Sun's local hour angle

        cosH = (cos(zenith) - (sinDec * sin(latitude))) / (cosDec * cos(latitude))

        if (cosH >  1)
          the sun never rises on this location (on the specified date)
        if (cosH < -1)
          the sun never sets on this location (on the specified date)

7b. finish calculating H and convert into hours

        if if rising time is desired:
          H = 360 - acos(cosH)
        if setting time is desired:
          H = acos(cosH)

        H = H / 15

8. calculate local mean time of rising/setting

        T = H + RA - (0.06571 * t) - 6.622

9. adjust back to UTC

        UT = T - lngHour
        NOTE: UT potentially needs to be adjusted into the range [0,24) by adding/subtracting 24

10. convert UT value to local time zone of latitude/longitude

        localT = UT + localOffset


phase of the moon?
sci.astro (astro = astronomy) newsgroup FAQ:

 John Horton Conway (the Princeton mathematician who is responsible for
 "the Game of Life") wrote a book with Guy and Berlekamp, _Winning
 Ways_, that describes in Volume 2 a number of useful calendrical
 rules.  One of these is an easy "in your head" algorithm for
 calculating the phase of the Moon, good to a day or better depending
 on whether you use his refinements or not.

 In the 20th century, calculate the remainder upon dividing the
 last two digits of the year by 19; if greater than 9, subtract
 19 from this to get a number between -9 and 9.

 Multiply the result by 11 and reduce modulo 30 to obtain a
 number between -29 and +29.

 Add the day of the month and the number of the month (except
 for Jan and Feb use 3 and 4 for the month number instead of
 1 and 2).

 Subtract 4.

 Reduce modulo 30 to get a number between 0 and 29. This is
 the age of the Moon.

 Example: What was the phase of the Moon on D-Day (June 6,
 1944)?

 Answer: 44/19=2 remainder 6.

 6*11=66, reduce modulo 30 to get 6.

 Add 6+6 to this and subtract 4: 6+6+6-4=14; the Moon was (nearly)
 full. I understand that the planners of D-day did care about the phase
 of the Moon, either because of illumination or because of tides. I
 think that Don Olsen recently discussed this in _Sky and Telescope_
 (within the past several years).

 In the 21st century use -8.3 days instead of -4 for the last number.

 Conway also gives refinements for the leap year cycle and also
 for the slight variations in the lengths of months; what I have
 given should be good to +/- a day or so.

************************************************************************** */

?>